"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = __importDefault(require("axios"));
const Cache_1 = __importDefault(require("../etc/Cache"));
const User_1 = __importDefault(require("./User"));
class Payment {
    /**
     * Cek semua metode pembayaran yang tersedia
     * @returns Metode pembayaran yang tersedia
     */
    static getPaymentMethods() {
        return __awaiter(this, void 0, void 0, function* () {
            const cache = Cache_1.default.get("paymentMethods");
            if (cache)
                return cache.data;
            const req = yield axios_1.default.get("https://api.trakteer.id/v2/fe/payment-methods");
            if (typeof req.data !== "object")
                throw new Error("Invalid response");
            if (!req.data.data)
                throw new Error(req.data.message);
            Cache_1.default.set("paymentMethods", req.data.data, 60000);
            return req.data.data;
        });
    }
    /**
     * Mendapatkan total biaya yang harus dibayar
     * @param paymentMethod Metode pembayaran
     * @param amount Jumlah yang harus dibayar
     * @param is_payment_fee_by_supporter Apakah biaya admin dibayar oleh supporter
     * @returns Total biaya yang harus dibayar
     */
    static getTotalAmount(paymentMethod_1, amount_1) {
        return __awaiter(this, arguments, void 0, function* (paymentMethod, amount, is_payment_fee_by_supporter = true) {
            const req = yield axios_1.default.post("https://api.trakteer.id/v2/fe/payment/total", {
                is_payment_fee_by_supporter,
                payment_method: paymentMethod,
                price: amount,
            });
            if (req.data.status === "error")
                throw new Error(req.data.message);
            return req.data.data;
        });
    }
    static generateNewToken() {
        return __awaiter(this, void 0, void 0, function* () {
            const cache = Cache_1.default.get("csrfCookies");
            if (cache)
                return cache.data;
            const req = yield axios_1.default.get("https://trakteer.id/BbayuGt", {
                withCredentials: true,
            });
            const csrfMatch = req.data.match(/(?<=\"csrf-token\" content=\")(.*?)(?=\")/);
            if (!csrfMatch)
                throw new Error("Failed to get csrf token");
            if (!req.headers["set-cookie"])
                throw new Error("Failed to get cookie");
            const cookie = req.headers["set-cookie"]
                .map((a) => a.split(";")[0])
                .join("; ");
            Cache_1.default.set("csrfCookies", {
                csrf: csrfMatch[0],
                cookie: cookie,
            }, 60 * 60 * 1000); // 1 hour
            return {
                csrf: csrfMatch[0],
                cookie: cookie,
            };
        });
    }
    /**
     * Membuat pembayaran
     * @param creatorId ID kreator
     * @param paymentMethod Metode pembayaran
     * @param amount Jumlah yang harus dibayar
     * @param config Konfigurasi pembayaran
     */
    static createPayment(creatorId, paymentMethod, amount, config) {
        return __awaiter(this, void 0, void 0, function* () {
            var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q;
            const paymentMethods = yield this.getPaymentMethods();
            if (paymentMethods instanceof Error)
                throw paymentMethods;
            if (!paymentMethods[paymentMethod])
                throw new Error("Invalid payment method");
            const userDetail = yield User_1.default.getUserDetails(creatorId);
            if (userDetail instanceof Error)
                throw userDetail;
            switch (paymentMethods[paymentMethod].gateway) {
                case "xendit":
                    {
                        const paymentMethodData = paymentMethods[paymentMethod];
                        if (paymentMethodData.min_price &&
                            amount <
                                paymentMethodData.min_price /
                                    parseInt(userDetail.active_unit.data.price)) {
                            throw new Error("Minimum donation for this method is " +
                                paymentMethodData.min_price);
                        }
                        if (paymentMethod === "ovo" && !(config === null || config === void 0 ? void 0 : config.ovo_phone))
                            throw new Error("OVO phone number is required for OVO payment method");
                        if (paymentMethod === "linkaja" && !(config === null || config === void 0 ? void 0 : config.linkaja_phone))
                            throw new Error("LinkAja phone number is required for LinkAja payment method");
                        if (paymentMethod === "jenius" && !(config === null || config === void 0 ? void 0 : config.cashtag))
                            throw new Error("Cashtag is required for Jenius payment method");
                        const payload = {
                            creator_id: creatorId,
                            display_name: (_a = config === null || config === void 0 ? void 0 : config.display_name) !== null && _a !== void 0 ? _a : "Seseorang",
                            form: "create-tip",
                            guest_email: (_b = config === null || config === void 0 ? void 0 : config.guest_email) !== null && _b !== void 0 ? _b : "notexist@not.ex.ist", // Email is mandatory, but fake email is fine
                            is_anonym: ((_c = config === null || config === void 0 ? void 0 : config.is_anonym) !== null && _c !== void 0 ? _c : false) ? "on" : "off",
                            is_private: ((_d = config === null || config === void 0 ? void 0 : config.is_private) !== null && _d !== void 0 ? _d : false) ? "on" : "off",
                            is_remember_next: "off",
                            is_showing_email: ((_e = config === null || config === void 0 ? void 0 : config.is_showing_email) !== null && _e !== void 0 ? _e : false) ? "on" : "off",
                            payment_method: paymentMethod,
                            quantity: amount,
                            support_message: (_f = config === null || config === void 0 ? void 0 : config.support_message) !== null && _f !== void 0 ? _f : "",
                            times: "once",
                            unit_id: userDetail.active_unit.data.id,
                            ovo_phone: (_g = config === null || config === void 0 ? void 0 : config.ovo_phone) !== null && _g !== void 0 ? _g : undefined,
                            linkaja_phone: (_h = config === null || config === void 0 ? void 0 : config.linkaja_phone) !== null && _h !== void 0 ? _h : undefined,
                            cashtag: (_j = config === null || config === void 0 ? void 0 : config.cashtag) !== null && _j !== void 0 ? _j : undefined,
                        };
                        const token = yield this.generateNewToken();
                        if (token instanceof Error)
                            throw token;
                        const req = yield axios_1.default
                            .post(`https://trakteer.id/pay/xendit/${paymentMethod}`, payload, {
                            withCredentials: true,
                            withXSRFToken: true,
                            headers: {
                                "X-Csrf-Token": token.csrf,
                                cookie: token.cookie,
                            },
                        })
                            .catch((e) => {
                            if (e.status === 500)
                                throw new Error("Failed to create payment, cek minimum donation" +
                                    JSON.stringify(payload, null, 2) +
                                    JSON.stringify(e, null, 2));
                            if (e.status === 401 && paymentMethod === "jenius")
                                throw new Error("Cashtag is invalid");
                            throw new Error(e);
                        });
                        if (req.data.checkout_url) {
                            return new URL(req.data.checkout_url);
                        }
                        else if (req.data.order_id) {
                            return new URL(`https://trakteer.id/payment-status/${req.data.order_id}`);
                        }
                        else
                            throw new Error("Failed to create payment" +
                                JSON.stringify(req.data, null, 2));
                    }
                    break;
                case "midtrans": {
                    const paymentMethodData = paymentMethods[paymentMethod];
                    if (paymentMethodData.min_price &&
                        amount <
                            paymentMethodData.min_price /
                                parseInt(userDetail.active_unit.data.price)) {
                        throw new Error("Minimum donation for this method is " +
                            paymentMethodData.min_price);
                    }
                    const payload = {
                        creator_id: creatorId,
                        display_name: (_k = config === null || config === void 0 ? void 0 : config.display_name) !== null && _k !== void 0 ? _k : "Seseorang",
                        form: "create-tip",
                        guest_email: (_l = config === null || config === void 0 ? void 0 : config.guest_email) !== null && _l !== void 0 ? _l : "notexist@not.ex.ist", // Email is mandatory, but fake email is fine
                        is_anonym: ((_m = config === null || config === void 0 ? void 0 : config.is_anonym) !== null && _m !== void 0 ? _m : false) ? "on" : "off",
                        is_private: ((_o = config === null || config === void 0 ? void 0 : config.is_private) !== null && _o !== void 0 ? _o : false) ? "on" : "off",
                        is_remember_next: "off",
                        is_showing_email: ((_p = config === null || config === void 0 ? void 0 : config.is_showing_email) !== null && _p !== void 0 ? _p : false) ? "on" : "off",
                        payment_method: paymentMethod,
                        quantity: amount,
                        support_message: (_q = config === null || config === void 0 ? void 0 : config.support_message) !== null && _q !== void 0 ? _q : "",
                        times: "once",
                        unit_id: userDetail.active_unit.data.id,
                    };
                    const token = yield this.generateNewToken();
                    if (token instanceof Error)
                        throw token;
                    const req = yield axios_1.default.post(`https://trakteer.id/pay/midtrans/${paymentMethod}`, payload, {
                        withCredentials: true,
                        withXSRFToken: true,
                        headers: {
                            "X-Csrf-Token": token.csrf,
                            cookie: token.cookie,
                        },
                    });
                    if (req.status === 200) {
                        // This will return the midtrans token
                        const reqToken = req.data;
                        // Get available channel
                        const reqChannel = yield axios_1.default.get(`https://app.midtrans.com/snap/v1/transactions/${reqToken}`);
                        const enabledPayments = reqChannel.data.enabled_payments
                            .filter((a) => a.status === "up")
                            .map((a) => a.type);
                        if (enabledPayments.length === 0)
                            throw new Error("No payment channel available");
                        const reqMidtrans = yield axios_1.default.post(`https://app.midtrans.com/snap/v2/transactions/${reqToken}/charge`, {
                            payment_type: enabledPayments[0],
                            promo_details: null,
                        }, {
                            withCredentials: true,
                            headers: {
                                "X-Csrf-Token": token.csrf,
                                cookie: token.cookie,
                            },
                        });
                        if (reqMidtrans.status === 200) {
                            return reqMidtrans.data;
                        }
                        else {
                            throw new Error("Failed to create payment" +
                                JSON.stringify(reqMidtrans.data));
                        }
                    }
                }
                case "trakteer":
                    throw new Error("Payment method not supported (require login)");
                default:
                    throw new Error("Payment method not supported");
            }
        });
    }
}
exports.default = Payment;
